/*
Igor, transforms DB Records into XML
Copyright (C) 1999  David Cancel

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

package com.synz.igor;

import java.io.*;
import java.sql.*;
import java.util.Hashtable;

import com.synz.igor.IgorConstants;
import com.synz.igor.printer.Printer;
import com.synz.igor.processor.Processor;

/**
 * The main Igor class.
 *
 * @author <a href="mailto:dave@synz.com">David Cancel</a>
 * @version Revision: 0.1 Date: 1999/06/02
 */
public class Igor implements IgorConstants {

    /**
    * This method selects *all* of the records from the tableName 
    * you supplied and returns them in an XML-like format.
    *   i.e.
    *       <(db_tableName)>
    *           <......> [all rows and columns contained in the specified table]
    *       </(db_tableName)>
    *
    * @param db_conn - A reference to a Connection
    * @param db_tableName - The name of the table to query
    * @return - StringBuffer containing contents
    */
    public static StringBuffer tableToXml(String db_tableName, Connection db_conn) {
        return Processor.tableToXml(db_tableName, db_conn);
    }

    /**
    * This method loops through the resultset provided and returns the result
    * in an XML-like format.
    *   i.e. this call, rsToXml("dogs", rs)
    *       would produce the following,
    *           <dogs>
    *               <......> [values contained in resultset]
    *           </dogs>
    *
    * @param setName - A reference to a sql resultset
    * @param db_rs - Sets name of beginning and closing <></> tags.
    * @return - StringBuffer containing contents
    */
    public static StringBuffer rsToXml(String setName, ResultSet db_rs) {
        return Processor.rsToXml(setName, db_rs);
    }
    
    /**
    * This method parses a template(see sample in the Example directory) and
	* replaces the variables with records from the DB ResultSet. Template variables
	* are of this form %table.column%. The ResultSet is iterated and placed into a hastable
	* in this format, tablename.columnname '|' value
	* When the template is parsed all variables are matched up against the hashtable,
	* if the template variable does not exist in the hastable 'null' is returned.
    *    
    * @param db_rs - A SQL ResultSet
    * @param templateFile - The location of the xml template
    * @return - StringBuffer containing contents
    */
    public static StringBuffer rsToXmlTemplate(String templateFile, ResultSet db_rs) {
        return Processor.rsToXmlTemplate(templateFile, db_rs);
    }

    /**
    * @return - name and version information
    */
    public String getVersionInfo() {
        return NAME + " " + VERSION;
    }
//
}