/*
Igor, transforms DB Records into XML
Copyright (C) 1999  David Cancel

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

package com.synz.igor.parser;

import java.io.*;
import java.sql.*;
import java.util.Hashtable;

import com.synz.igor.IgorConstants;
import com.synz.igor.printer.Printer;

/**
 * This class is responsible for parsing ResultSets.
 *
 * @author <a href="mailto:dave@synz.com">David Cancel</a>
 * @version Revision: 0.1 Date: 1999/06/02
 */
public class ResultSetParser implements IgorConstants {

    /** 
    * Reads in the ResultSet and returns it as a Hashtable.
    * This method should be called for each row in the ResultSet.
    * i.e.
    *   do while not db_rs.eof
    *       ResultSetParser rsp = new ResultSetParser();
    *       rsp.parseRS(db_rs);
    *   next
    *
    * @return Hashtable - a Hashtable containing the column names 
    * and values of the ResultSet
    */
    public static Hashtable parseRS(ResultSet rs) {
        if (rs == null) {
            throw new RuntimeException("ResultSetParser.parseRS: ResultSet was null");
        }
        try {
            Hashtable hashtable = new Hashtable();
            ResultSetMetaData rsmd = rs.getMetaData();
            int numCols = rsmd.getColumnCount();
                // loop through columns
                Printer.Debug("ResultSetParser: Start For Loop ------>");
                for (int i = 1; i <= numCols; i++) {
                    String s = rsmd.getTableName(i).toLowerCase() + "." + rsmd.getColumnLabel(i).toLowerCase();
                    String r = rs.getString(i);
                    Printer.Debug("ResultSetParser: hashtable.put(" + s + ", " + r + ")");
                    hashtable.put(s, r);
                }
            Printer.Debug("ResultSetParser: End For Loop ------>\n");
            return hashtable;
        } catch (SQLException e) {Printer.Err("ResultSetParser: ", e);}
    return null;
    }


    /** 
    * Test Harness 
    */
    public static void main(String arg[]) {
/*
        Connection Conn = null; 
        try {
            Class.forName(DB_DRIVER).newInstance();
            Conn = DriverManager.getConnection(DB_STRING);
            Printer.Debug("ResultSetParser: Connection succesful.");
            Statement db_stmt = Conn.createStatement();
            ResultSet db_rs = db_stmt.executeQuery("Select people.name,cars.modelname,cars.modelyear from people,cars where people.id = cars.id");

            while (db_rs.next()) {
                // will print out results only if DEBUG flag
                // in IgorConstants is set to true
                parseRS(db_rs);
            }
        }
        catch (Exception e) {Printer.Err("ResultSetParser: Test Harness error ", e);}
        finally {
            if (Conn != null) {
                Printer.Debug("ResultSetParser: Closing connection.");
                try {Conn.close();} catch (Exception e) {Printer.Err("ResultSetParser: Test Harness error ", e);}
            }
        }
*/
    }
  
//
}