/*
Igor, transforms DB Records into XML
Copyright (C) 1999  David Cancel

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

package com.synz.igor.parser;

import java.io.*;
import java.util.Hashtable;

import com.synz.igor.IgorConstants;
import com.synz.igor.printer.Printer;

/**
 * This class is responsible for parsing templates.
 *
 * @author <a href="mailto:dave@synz.com">David Cancel</a>
 * @version Revision: 0.1 Date: 1999/06/02999/05/25
 */
public class TemplateParser implements IgorConstants {

    // all static methods, shouldn't use this.
    private TemplateParser() {}

    /** 
    * Reads in the template file and returns it as a
    * StringBuffer.
    *
    * @return StringBuffer - the contents of the template file
    */
    public static StringBuffer parseTemplate(String templateFile)
        throws IOException {
            StringBuffer buf = new StringBuffer();
            BufferedReader in;
            String s = new String();
            
            // load template file
            File tFile = new File(templateFile);
            if (tFile.canRead()) {
                try {
                    Printer.Debug("Parser: Loading " + tFile);
                    in = new BufferedReader(new FileReader(templateFile));
                    while ((s = in.readLine()) != null) {
                        buf.append(s);
                        buf.append("\n");
                    }
                    in.close();
                    Printer.Debug("TemplateParser: buf = \n" + buf.toString());
                    return buf;
                } catch (Exception e) {
                    Printer.Err("Parser: Could not load template: " + tFile, e);
                } 
            } else {throw new IOException("Could not find/load templateFile");}
            return null;
    }

    /** 
    * Reads in the template file and returns it as a
    * StringBuffer.
    *
    * @param String s - the string containing the macros to be replaced
    * @param char c - delimeter used, default is '%'
    * @param Hashtable hashtable - Hashtable of the name value pairs to be replaced in template
    * @return StringBuffer - the contents of the template file
    */
    public static String replaceMacros(String s, char c, Hashtable hashtable) {
        StringBuffer stringbuffer = new StringBuffer();
        int i = 0;
        int k = s.length();
        do {
            int j = findSingleChar(s, i, k, c);
            stringbuffer.append(removeDoubleChar(s.substring(i, j), c));
            if(j >= k)
                break;
            //Printer.Debug("substituteMacro("+stringbuffer+", "+s+", "+c+", "+hashtable+", "+j+", "+k);                
            i = substituteMacro(stringbuffer, s, c, hashtable, j, k);
        }
        while(i < k);
        return stringbuffer.toString();
    }

    private static int findSingleChar(String s, int i, int j, char c) {
        boolean flag = false;
        int k = j;
        while(!flag) {
            k = s.indexOf(c, i);
            if(k == -1)
                return j;
            if(s.charAt(k + 1) == c)
                i = k + 2;
            else
                flag = true;
        }
        return k;
    }
    
    private static String removeDoubleChar(String s, char c) {
        char ac[] = s.toCharArray();
        int i = ac.length;
        int j = 0;
        Character character = new Character(c);
        String s1 = character.toString() + character.toString();
        do {
            int k = s.indexOf(s1, j);
            if(k == -1)
                return new String(ac, 0, i);
            i--;
            System.arraycopy(ac, k + 1, ac, k, i - k);
            j = k + 1;
        }
        while(true);
    }

    private static int substituteMacro(StringBuffer stringbuffer, 
                                String s, char c, Hashtable hashtable, int i, int j) {
        int k = 0;
        int l = 0;
        boolean flag = false;
        char ac[] = s.toCharArray();
        StringBuffer stringbuffer1 = new StringBuffer(255);
        for(l = i + 1; l < j && (k > 0 || ac[l] != c) && !flag;) {
            if(ac[l] == '(' && ac[l - 1] != '\\')
                k++;
            if(ac[l] == ')' && ac[l - 1] != '\\' && --k == 0)
                flag = true;
            stringbuffer1.append(ac[l++]);
        }
        //Printer.Debug("hashtable.get(stringbuffer1.toString()) " + hashtable.get(stringbuffer1.toString()));
        stringbuffer.append(hashtable.get(stringbuffer1.toString()));
        if(l < j && ac[l] == c)
            l++;
        return l;
    }

//
}