/*
Igor, transforms DB Records into XML
Copyright (C) 1999  David Cancel

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

package com.synz.igor.processor;

import java.io.*;
import java.sql.*;
import java.util.Hashtable;

import com.synz.igor.parser.*;
import com.synz.igor.IgorConstants;
import com.synz.igor.printer.Printer;



/**
 * This class handles the processing of ResultSets into XML.
 *
 * @author <a href="mailto:dave@synz.com">David Cancel</a>
 * @version Revision: 0.1 Date: 1999/06/02
 */
public class Processor implements IgorConstants {


    /**
    * @param db_conn A reference to a Connection
    * @param db_tableName The name of the table to query
    */
    public static StringBuffer tableToXml(String db_tableName, Connection db_conn) {
        if (db_conn == null) {
            throw new RuntimeException("Processor: Connection argument was null.");
        }
        if (db_tableName == null || db_tableName.length() <= 0) {
            throw new RuntimeException("Processor: Illegal db_tableName argument.");
        }
        
        int i;
        int rnumber = 1;
        Statement db_stmt = null;
        ResultSet db_rs = null;
        StringBuffer db_sql = new StringBuffer();
        StringBuffer buf = new StringBuffer();
        
        try {
            db_stmt = db_conn.createStatement();
            Printer.Debug("Processor: Created statement.");
            
            // create sql
            db_sql.append("Select * From ");
            db_sql.append(db_tableName);
            Printer.Debug("Processor: db_sql = " + db_sql.toString());
            
            // loop through resultset
            db_rs = db_stmt.executeQuery(db_sql.toString());
                ResultSetMetaData rsmd = db_rs.getMetaData();
                int numCols = rsmd.getColumnCount();
                
                while (db_rs.next()) {
                    buf.append("<");
                    buf.append(db_tableName.toLowerCase());
                    buf.append(">\n");
                        for (i = 1; i <= numCols; i++) {
                            buf.append("    <");
                            buf.append(rsmd.getColumnLabel(i).toLowerCase());
                            buf.append(">");
                            buf.append(db_rs.getString(i));
                            buf.append("</");
                            buf.append(rsmd.getColumnLabel(i).toLowerCase());
                            buf.append(">\n");
                        }
                    buf.append("</");
                    buf.append(db_tableName.toLowerCase());
                    buf.append(">\n");
                }
                return buf;
        }
        catch (SQLException e) {Printer.Err("Processor: Error connecting to database.", e);}
	    finally {
            try {
                if(db_stmt != null) {
                    Printer.Debug("Processor: Closing statement.");
                    db_stmt.close();
                }
            }
            catch (SQLException e) {Printer.Err("Processor: Error closing statement.", e);};
        }        
        return buf;
    }

    /**
    * @param setName - A reference to a sql resultset
    * @param db_rs - Sets name of beginning and closing <></> tags.
    */
    public static StringBuffer rsToXml(String setName, ResultSet rs) {
        if (rs == null) {
            throw new RuntimeException("Processor: ResultSet argument was null.");
        }
        if (setName == null || setName.length() <= 0) {
            throw new RuntimeException("Processor: Illegal setName argument.");
        }        
        
        int i;
        int rnumber = 1;
        StringBuffer buf = new StringBuffer();
        
        try {
            // loop through resultset
            ResultSetMetaData rsmd = rs.getMetaData();
            int numCols = rsmd.getColumnCount();
                
                while (rs.next()) {
                    buf.append("<");
                    buf.append(setName.toLowerCase());
                    buf.append(">\n");
                        for (i = 1; i <= numCols; i++) {
                            buf.append("    <");
                            buf.append(rsmd.getColumnLabel(i).toLowerCase());
                            buf.append(">");
                            buf.append(rs.getString(i));
                            buf.append("</");
                            buf.append(rsmd.getColumnLabel(i).toLowerCase());
                            buf.append(">\n");
                        }
                    buf.append("</");
                    buf.append(setName.toLowerCase());
                    buf.append(">\n");
                }
                return buf;
        }
        catch (SQLException e) {Printer.Err("Processor: Error connecting to database.", e);}
	    finally {
            try {
                if(rs != null) {
                    Printer.Debug("Processor: Closing resultset.");
                    rs.close();
                }
            }
            catch (SQLException e) {Printer.Err("Processor: Error closing statement. ", e);};
        }        
        return buf;
    }
    
    /**
    * @param rs A SQL ResultSet
    * @param templateFile The location of the xml template
    */
    public static StringBuffer rsToXmlTemplate(String templateFile, ResultSet rs) {
        StringBuffer buf = new StringBuffer();
        Hashtable hash = new Hashtable();
        String s = null;
            if (templateFile != null || rs != null) {
                try {
                    s = TemplateParser.parseTemplate(templateFile).toString().trim();
                    if (s != null) {
                        hash = ResultSetParser.parseRS(rs);
                        if (hash != null) {
                            buf.append(TemplateParser.replaceMacros(s, '%', hash));    
                        } else {throw new Exception("Processor: hash = null");}
                    }
                } catch (Exception e) {Printer.Err("Processor: Error reading template. ", e);}
            }
        return buf;
    }
    
    /** Test Harness */
    public static void main(String arg[]) {
/*
        Connection Conn = null; 
        try {
            Class.forName(DB_DRIVER).newInstance();
            Conn = DriverManager.getConnection(DB_STRING);
            Printer.Debug("Processor: Connection succesful.");
            System.out.println(tableToXml("people" ,Conn).toString());

            Statement db_stmt = Conn.createStatement();
            ResultSet db_rs = db_stmt.executeQuery("Select people.name,people.email,cars.modelname,cars.modelyear from people,cars where people.id = cars.id");
            System.out.println(rsToXml("testname",db_rs).toString());
            while (db_rs.next()) {
                System.out.println(rsToXmlTemplate("C:\\dcancel\\projects\\synz\\htdocs\\projects\\igor\\examples\\cars.txml", db_rs));
            }
            
        }
        catch (Exception e) {Printer.Err("Processor: Test Harness error ", e);}
        finally {
            if (Conn != null) {
                Printer.Debug("Processor: Closing connection.");
                try {Conn.close();} catch (Exception e) {Printer.Err("Processor: Test Harness error ", e);}
            }
        }
*/

    }
//
}

